"use client";

import axios from "axios";
import * as React from 'react';
import { InfoIcon, MoreHorizontal, Plus, Save, Tags, Trash } from "lucide-react"
import { useEffect, useState } from "react";
import { useSession } from "next-auth/react";
import { Button } from "@/components/ui/button"
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem, CommandList } from "@/components/ui/command"
import { Input } from "@/components/ui/input";
import { useRouter } from "next/navigation";
import { useForm } from "react-hook-form";
import { Form, FormControl, FormField, FormItem, FormMessage } from "@/components/ui/form";
import * as z from "zod";
import { zodResolver } from "@hookform/resolvers/zod";
import { DropdownMenu, DropdownMenuContent, DropdownMenuGroup, DropdownMenuItem, DropdownMenuLabel, DropdownMenuSeparator, DropdownMenuShortcut, DropdownMenuSub, DropdownMenuSubContent, DropdownMenuSubTrigger, DropdownMenuTrigger } from "@/components/ui/dropdown-menu";
import { Label } from "@/components/ui/label";
import { ToastAction } from "@/components/ui/toast"
import { useToast } from "@/components/ui/use-toast"
import InfoNotice from "@/components/elements/info-notice";
import { Toaster } from "@/components/ui/toaster";
import { stringifyError } from "next/dist/shared/lib/utils";


const TTSFiltersPage = () => {
    const { data: session, status } = useSession();
    const [moreOpen, setMoreOpen] = useState(0)
    const [tag, setTag] = useState("blacklisted")
    const [open, setOpen] = useState(false)
    const [userTags, setUserTag] = useState<{ username: string, tag: string }[]>([])
    const { toast } = useToast()
    const [error, setError] = useState("")
    const router = useRouter();

    const tags = [
        "blacklisted",
        "priority"
    ]

    const toasting = (title: string, error: Error) => {
        toast({
            title: title,
            description: error.message,
            variant: "error"
        })
    }

    const success = (title: string, description: string) => {
        toast({
            title: title,
            description: description,
            variant: "success"
        })
    }

    // Username blacklist
    const usernameFilteredFormSchema = z.object({
        username: z.string().trim().min(4).max(25).regex(new RegExp("[a-zA-Z0-9][a-zA-Z0-9\_]{3,24}"), "Must be a valid twitch username."),
        tag: z.string().trim()
    });

    const usernameFilteredForm = useForm({
        resolver: zodResolver(usernameFilteredFormSchema),
        defaultValues: {
            username: "",
            tag: ""
        }
    });

    useEffect(() => {
        const fetchData = async () => {
            try {
                const userFiltersData = await axios.get("/api/settings/tts/filter/users")
                setUserTag(userFiltersData.data ?? [])
            } catch (error) {
                toasting("Failed to fetch all the username filters.", error as Error)
            }

            try {
                const replacementData = await axios.get("/api/settings/tts/filter/words")
                setReplacements(replacementData.data ?? [])
            } catch (error) {
                toasting("Failed to fetch all the word filters.", error as Error)
            }
        };

        fetchData().catch((error) => toasting("Failed to fetch all the username filters.", error as Error));
    }, []);

    const onDelete = () => {
        const username = userTags[Math.log2(moreOpen)].username
        axios.delete("/api/settings/tts/filter/users?username=" + username)
            .then(() => {
                setUserTag(userTags.filter((u) => u.username != username))
                success("Username filter deleted", `"${username.toLowerCase()}" is now back to normal.`)
            }).catch((error) => toasting("Failed to delete the username filter.", error as Error))
    }

    const isSubmitting = usernameFilteredForm.formState.isSubmitting;

    const onAddExtended = (values: z.infer<typeof usernameFilteredFormSchema>, test: boolean = true) => {
        const original = userTags.find(u => u.username.toLowerCase() == values.username.toLowerCase())

        if (test)
            values.tag = tag

        axios.post("/api/settings/tts/filter/users", values)
            .then((d) => {
                if (original == null) {
                    userTags.push({ username: values.username.toLowerCase(), tag: values.tag })
                } else {
                    original.tag = values.tag
                }
                setUserTag(userTags)

                usernameFilteredForm.reset();
                router.refresh();
                if (values.tag == "blacklisted")
                    success("Username filter added", `"${values.username.toLowerCase()}" will be blocked.`)
                else if (values.tag == "priority")
                    success("Username filter added", `"${values.username.toLowerCase()}" will be taking priority.`)
            }).catch(error => toasting("Failed to add the username filter.", error as Error))
    }

    const onAdd = (values: z.infer<typeof usernameFilteredFormSchema>) => {
        onAddExtended(values, true)
    }

    // Word replacement
    const [replacements, setReplacements] = useState<{ id: string, search: string, replace: string, userId: string }[]>([])

    const onReplaceAdd = async () => {
        if (search.length <= 0) {
            toasting("Unable to add the word filter.", new Error("Search must not be empty."))
            return
        }

        await axios.post("/api/settings/tts/filter/words", { search, replace })
            .then(d => {
                replacements.push({ id: d.data.id, search: d.data.search, replace: d.data.replace, userId: d.data.userId })
                setReplacements(replacements)
                setSearch("")
                success("Word filter added", `"${d.data.search}" will be replaced.`)
            }).catch(error => toasting("Failed to add the word filter.", error as Error))
    }

    const onReplaceUpdate = async (data: { id: string, search: string, replace: string, userId: string }) => {
        await axios.put("/api/settings/tts/filter/words", data)
            .then(() => success("Word filter updated", ""))
            .catch(error => toasting("Failed to update the word filter.", error as Error))
    }

    const onReplaceDelete = async (id: string) => {
        await axios.delete("/api/settings/tts/filter/words?id=" + id)
            .then(d => {
                const r = replacements.filter(r => r.id != d.data.id)
                setReplacements(r)
                success("Word filter deleted", `No more filter for "${d.data.search}"`)
            }).catch(error => toasting("Failed to delete the word filter.", error as Error))
    }

    let [search, setSearch] = useState("")
    let [replace, setReplace] = useState("")
    let [searchInfo, setSearchInfo] = useState("")

    return (
        // <div>
        //   <div className="text-2xl text-center pt-[50px]">TTS Filters</div>
        //   <div className="px-10 py-1 w-full h-full flex-grow inset-y-1/2">
        //     <InfoNotice message="You can tag certain labels to twitch users, allowing changes applied specifically to these users when using the text to speech feature." hidden={false} />
        //     <div>
        //       {userTags.map((user, index) => (
        //         <div key={user.username + "-tags"} className="flex w-full items-start justify-between rounded-md border px-4 py-2 mt-2">
        //           <p className="text-base font-medium">
        //             <span className="mr-2 rounded-lg bg-primary px-2 py-1 text-xs text-primary-foreground">
        //               {user.tag}
        //             </span>
        //             <span className="text-white">{user.username}</span>
        //           </p>
        //           <DropdownMenu open={(moreOpen & (1 << index)) > 0} onOpenChange={() => setMoreOpen(v => v ^ (1 << index))}>
        //             <DropdownMenuTrigger asChild>
        //               <Button variant="ghost" size="xs" className="bg-purple-500 hover:bg-purple-600">
        //                 <MoreHorizontal className="h-4 w-4" />
        //               </Button>
        //             </DropdownMenuTrigger>
        //             <DropdownMenuContent align="end" className="w-[200px] bg-popover">
        //               <DropdownMenuLabel>Actions</DropdownMenuLabel>
        //               <DropdownMenuGroup>
        //                 <DropdownMenuSub>
        //                   <DropdownMenuSubTrigger>
        //                     <Tags className="mr-2 h-4 w-4" />
        //                     Apply label
        //                   </DropdownMenuSubTrigger>
        //                   <DropdownMenuSubContent className="p-0">
        //                     <Command>
        //                       <CommandInput
        //                         placeholder="Filter label..."
        //                         autoFocus={true}
        //                       />
        //                       <CommandList>
        //                         <CommandEmpty>No label found.</CommandEmpty>
        //                         <CommandGroup>
        //                           {tags.map((tag) => (
        //                             <CommandItem
        //                               key={user.username + "-tag"}
        //                               value={tag}
        //                               onSelect={(value) => {
        //                                 onAddExtended({ username: userTags[index].username, tag: value}, false)
        //                                 setMoreOpen(0)
        //                               }}
        //                             >
        //                               {tag}
        //                             </CommandItem>
        //                           ))}
        //                         </CommandGroup>
        //                       </CommandList>
        //                     </Command>
        //                   </DropdownMenuSubContent>
        //                 </DropdownMenuSub>
        //                 <DropdownMenuSeparator />
        //                 <DropdownMenuItem key={user.username + "-delete"} onClick={onDelete} className="text-red-600">
        //                   <Trash className="mr-2 h-4 w-4" />
        //                   Delete
        //                 </DropdownMenuItem>
        //               </DropdownMenuGroup>
        //             </DropdownMenuContent>
        //           </DropdownMenu>
        //         </div>
        //       ))}
        //       <Form {...usernameFilteredForm}>
        //         <form onSubmit={usernameFilteredForm.handleSubmit(onAdd)}>
        //           <div className="flex w-full items-center justify-between rounded-md border px-4 py-2 gap-3 mt-2">
        //             <Label className="rounded-lg bg-primary px-2 py-1 text-xs text-primary-foreground">
        //               {tag}
        //             </Label>
        //             <FormField
        //               control={usernameFilteredForm.control}
        //               name="username"
        //               render={({ field }) => (
        //                 <FormItem key={"new-username"} className="flex-grow">
        //                   <FormControl>
        //                     <Input id="username" placeholder="Enter a twitch username" {...field} />
        //                   </FormControl>
        //                   <FormMessage />
        //                 </FormItem>
        //               )}
        //             />
        //             <Button variant="ghost" size="sm" type="submit" className="bg-green-500 hover:bg-green-600 items-center align-middle" disabled={isSubmitting}>
        //               <Plus className="h-6 w-6" />
        //             </Button>
        //             <DropdownMenu open={open} onOpenChange={setOpen}>
        //               <DropdownMenuTrigger asChild>
        //                 <Button size="sm" {...usernameFilteredForm} className="bg-purple-500 hover:bg-purple-600" disabled={isSubmitting}>
        //                   <MoreHorizontal className="h-6 w-6" />
        //                 </Button>
        //               </DropdownMenuTrigger>
        //               <DropdownMenuContent align="end" className="w-[200px] bg-popover">
        //                 <DropdownMenuLabel>Actions</DropdownMenuLabel>
        //                 <DropdownMenuGroup>
        //                   <DropdownMenuSub>
        //                     <DropdownMenuSubTrigger>
        //                       <Tags className="mr-2 h-4 w-4" />
        //                       Apply label
        //                     </DropdownMenuSubTrigger>
        //                     <DropdownMenuSubContent className="p-0">
        //                       <Command>
        //                         <CommandInput
        //                           placeholder="Filter label..."
        //                           autoFocus={true}
        //                         />
        //                         <CommandList>
        //                           <CommandEmpty>No label found.</CommandEmpty>
        //                           <CommandGroup>
        //                             {tags.map((tag) => (
        //                               <CommandItem
        //                                 value={tag}
        //                                 key={tag + "-tag"}
        //                                 onSelect={(value) => {
        //                                   setTag(value)
        //                                   setOpen(false)
        //                                 }}
        //                               >
        //                                 {tag}
        //                               </CommandItem>
        //                             ))}
        //                           </CommandGroup>
        //                         </CommandList>
        //                       </Command>
        //                     </DropdownMenuSubContent>
        //                   </DropdownMenuSub>
        //                 </DropdownMenuGroup>
        //               </DropdownMenuContent>
        //             </DropdownMenu>
        //           </div>
        //         </form>
        //       </Form>
        //     </div>
        <div>
            <div>

                <div>
                    <p className="text-center text-2xl text-white pt-[80px]">Regex Replacement</p>
                    <div>
                        {replacements.map((term: { id: string, search: string, replace: string, userId: string }) => (
                            <div key={term.id} className="flex flex-row w-full items-start justify-between rounded-lg border px-4 py-3 gap-3 mt-[15px]">
                                <Input id="search" placeholder={term.search} className="flex" onChange={e => term.search = e.target.value} defaultValue={term.search} />
                                <Input id="replace" placeholder={term.replace} className="flex" onChange={e => term.replace = e.target.value} defaultValue={term.replace} />
                                <Button className="bg-blue-500 hover:bg-blue-600 items-center align-middle" onClick={_ => onReplaceUpdate(term)}>
                                    <Save className="h-4 w-4" />
                                </Button>
                                <Button className="bg-red-500 hover:bg-red-600 items-center align-middle" onClick={_ => onReplaceDelete(term.id)}>
                                    <Trash className="h-4 w-4" />
                                </Button>
                            </div>
                        ))}
                        <div className="flex flex-row w-full items-center justify-center rounded-lg border px-3 py-3 mt-[15px]">
                            <div className="flex flex-col flex-grow">
                                <div className="flex flex-row w-full items-center justify-center gap-3">
                                    <Input id="search" placeholder="Enter a term to search for" onChange={e => {
                                        setSearch(e.target.value);
                                        try {
                                            new RegExp(e.target.value)
                                            setSearchInfo("Valid regular expression.")
                                        } catch (e) {
                                            setSearchInfo("Invalid regular expression. Regular search will be used instead.")
                                        }
                                    }} />
                                    <Input id="replace" placeholder="Enter a term to replace with" onChange={e => setReplace(e.target.value)} />
                                    <Button className="bg-green-500 hover:bg-green-600 items-center align-middle" onClick={onReplaceAdd}>
                                        <Plus className="h-4 w-4" />
                                    </Button>
                                </div>
                                <div className={searchInfo.length == 0 ? "hidden" : ""}>
                                    <InfoIcon className="inline-block h-4 w-4" />
                                    <p className="inline-block text-orange-400 text-sm pl-[7px]">{searchInfo}</p>
                                </div>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            <Toaster />
        </div>
    );
}

export default TTSFiltersPage;