import { db } from "@/lib/db"
import { NextResponse } from "next/server";
import fetchUserWithImpersonation from "@/lib/fetch-user-impersonation";

export async function GET(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return new NextResponse("Unauthorized", { status: 401 });
        }

        const filters = await db.ttsWordFilter.findMany({
            where: {
                userId: user.id
            }
        });

        return NextResponse.json(filters);
    } catch (error) {
        console.log("[TTS/FILTER/WORDS]", error);
        return new NextResponse("Internal Error", { status: 500 });
    }
}

export async function POST(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return new NextResponse("Unauthorized", { status: 401 });
        }

        const { search, replace } = await req.json();
        if (!search || search.length < 4 || search.length > 200) return new NextResponse("Bad Request", { status: 400 });
        if (replace == null) return new NextResponse("Bad Request", { status: 400 });

        const filter = await db.ttsWordFilter.create({
          data: {
            search,
            replace,
            userId: user.id
          }
        });

        return NextResponse.json(filter);
    } catch (error) {
        console.log("[TTS/FILTER/WORDS]", error);
        return new NextResponse("Internal Error", { status: 500 });
    }
}

export async function PUT(req: Request) {
  try {
      const user = await fetchUserWithImpersonation(req)
      if (!user) {
          return new NextResponse("Unauthorized", { status: 401 });
      }

      const { id, search, replace } = await req.json();
      if (!id || id.length < 1) return new NextResponse("Bad Request", { status: 400 });
      if (!search || search.length < 4 || search.length > 200) return new NextResponse("Bad Request", { status: 400 });
      if (replace == null) return new NextResponse("Bad Request", { status: 400 });

      const filter = await db.ttsWordFilter.update({
        where: {
          id
        },
        data: {
          search,
          replace,
          userId: user.id
        }
      });

      return NextResponse.json(filter);
  } catch (error) {
      console.log("[TTS/FILTER/WORDS]", error);
      return new NextResponse("Internal Error", { status: 500 });
  }
}

export async function DELETE(req: Request) {
    try {
        const user = await fetchUserWithImpersonation(req)
        if (!user) {
            return new NextResponse("Unauthorized", { status: 401 });
        }

        const { searchParams } = new URL(req.url)
        const id = searchParams.get('id') as string
        const search = searchParams.get('search') as string
        if (!id && !search) return new NextResponse("Bad Request", { status: 400 });

        if (search) {
          if (search.length < 4 || search.length > 200) return new NextResponse("Bad Request", { status: 400 });

          const filter = await db.ttsWordFilter.delete({
            where: {
              userId_search: {
                userId: user.id,
                search
              }
            }
          });

          return NextResponse.json(filter)
        } else if (id) {
          if (id.length < 1) return new NextResponse("Bad Request", { status: 400 });

          const filter = await db.ttsWordFilter.delete({
            where: {
              id: id
            }
          });

          return NextResponse.json(filter)
        }
        return new NextResponse("Bad Request", { status: 400 });
    } catch (error) {
        console.log("[TTS/FILTER/WORDS]", error);
        return new NextResponse("Internal Error" + error, { status: 500 });
    }
}